WhoRoll = LibStub("AceAddon-3.0"):NewAddon("WhoRoll", "AceEvent-3.0")

local WhoRoll = WhoRoll

local deformat = LibStub("LibDeformat-3.0")

local currentLoots = {}
WhoRoll.currentLoots = currentLoots

-- globals
local _G = _G
local GREED = _G.GREED
local NEED = _G.NEED
local PASS = _G.PASS
local DISENCHANT = "DISENCHANT"

local GetItemInfo = _G.GetItemInfo
local GetLootRollItemLink = _G.GetLootRollItemLink
local GetLootRollItemInfo = _G.GetLootRollItemInfo

local function debug(msg)
	--[===[@debug@
	ChatFrame4:AddMessage(msg)
	--@end-debug@]===]
end

function WhoRoll:OnEnable()
	self:RegisterEvent("START_LOOT_ROLL")
	self:RegisterEvent("CHAT_MSG_LOOT")
	self:UpdateTooltip()
end

function WhoRoll:START_LOOT_ROLL(_, rollId, rollTime)
	local link = GetLootRollItemLink(rollId)
	debug("START_LOOT_ROLL: .. " .. rollId .. "... " .. link)

	if not currentLoots[link] then
		newLootWatch = {}
		newLootWatch.rollId = rollId
		newLootWatch.rolls = {}
		currentLoots[link] = newLootWatch
	end
	self:UpdateTooltip()
end

local playerName = UnitName("player")
local parsers = {
	[LOOT_ROLL_GREED_SELF] = function(message, pattern)
			return playerName, deformat(message, pattern), GREED
		end,
	[LOOT_ROLL_GREED] = function(message, pattern)
			local name, item = deformat(message, pattern)
			return name, item, GREED
		end,
	[LOOT_ROLL_NEED_SELF] = function(message, pattern)
			return playerName, deformat(message, pattern), NEED
		end,
	[LOOT_ROLL_NEED] = function(message, pattern)
			local name, item = deformat(message, pattern)
			return name, item, NEED
		end,
	[LOOT_ROLL_DISENCHANT_SELF] = function(message, pattern)
			return playerName, deformat(message, pattern), DISENCHANT
		end,
	[LOOT_ROLL_DISENCHANT] = function(message, pattern)
			local name, item = deformat(message, pattern)
			return name, item, DISENCHANT
		end,
	[LOOT_ROLL_PASSED_SELF] = function(message, pattern)
			return playerName, deformat(message, pattern), PASS
		end,
	[LOOT_ROLL_PASSED] = function(message, pattern)
			local name, item = deformat(message, pattern)
			return name, item, PASS
		end,
}

parsers[LOOT_ROLL_PASSED_SELF_AUTO] = parsers[LOOT_ROLL_PASSED_SELF]
parsers[LOOT_ROLL_PASSED_AUTO] = parsers[LOOT_ROLL_PASSED]
parsers[LOOT_ROLL_PASSED_AUTO_FEMALE] = parsers[LOOT_ROLL_PASSED]

local matchers = {}
for k in pairs(parsers) do
	matchers[k] = k:gsub("%%s", ".+")
end
_G.matchers = matchers

local matcher_LOOT_ROLL_WON = LOOT_ROLL_WON:gsub("%%s", ".+")
local matcher_LOOT_ROLL_ALL_PASSED = LOOT_ROLL_ALL_PASSED:gsub("%%s", ".+")

local function trackRoll(name, itemLink, selection)
	debug("track Roll: " .. (name or "nil") .. " selected on " .. (itemLink or "nil"))
	if not currentLoots[itemLink] then
		currentLoots[itemLink] = {}
		currentLoots[itemLink].rolls = {}
	end
	local thisLoot = currentLoots[itemLink]
	thisLoot.rolls[name] = selection
end

local function removeRoll(itemLink)
	if not currentLoots[itemLink] then return end
	wipe(currentLoots[itemLink])
	currentLoots[itemLink] = nil
end

function WhoRoll:CHAT_MSG_LOOT(_, message)
	local itemLink, name, selection

	for k,v in pairs(matchers) do
		if message:match(v) then
			local name, itemLink, selection = parsers[k](message, k)
			trackRoll(name, itemLink, selection)
			self:UpdateTooltip()
			return
		end
	end

	if deformat(message,LOOT_ROLL_WON) then
		_, itemLink = deformat(message,LOOT_ROLL_WON)
		removeRoll(itemLink)
	elseif deformat(message, LOOT_ROLL_ALL_PASSED) then
		itemLink = deformat(message,LOOT_ROLL_ALL_PASSED)
		removeRoll(itemLink)
	else
		--didn't match anything
		debug("string could not be matched")
		debug(message)
		return
	end
	self:UpdateTooltip();
end
